{*****************************************************************************}
{                                                                             }
{  llDatabase.pas                                                             }
{                                                                             }
{  Database and DataSet utility routines                                      }
{                                                                             }
{  See LogicLib.dpk for copyright/licensing information.                      }
{                                                                             }
{*****************************************************************************}

unit llDatabase;

interface

uses
  SysUtils, Classes, DB, DBClient;

procedure CloneDataset(ASource: TDataSet; ATarget: TClientDataSet;
                       const AFieldDefsOnly: Boolean = False);

procedure CopyRecord(ASource, ATarget: TDataSet);

implementation

{+}
procedure CloneDataset(ASource: TDataset; ATarget: TClientDataSet;
                       const AFieldDefsOnly: Boolean = False);
{
  Copies the field definitions and optionally the data from a TDataSet
  descendant into a TClientDataSet.

  The data is copied from the current position in the source dataset until Eof,
  in case the source is using a forward-only cursor.  Pre-position the source
  dataset using First if you want all of the data copied.

  The ClientDataSet has LogChanges set to False upon return from this, so if
  you want to use the change log, turn that back on afterwards.
-}
var
  i: Integer;
begin
  if ATarget.State <> dsInactive then begin
    ATarget.EmptyDataSet;
    ATarget.Close;
  end;
  ATarget.FieldDefs.Clear;
  for i := 0 to ASource.FieldDefs.Count - 1 do
    ATarget.FieldDefs.Add(ASource.FieldDefs[i].Name,
                          ASource.FieldDefs[i].DataType,
                          ASource.FieldDefs[i].Size,
                          ASource.FieldDefs[i].Required);
  ATarget.CreateDataSet;
  ATarget.LogChanges := False;
  if AFieldDefsOnly then
    Exit;
  // ASource.First;   Can't do that on a forward-only cursor!
  while not ASource.Eof do begin
    ATarget.Append;
    for i := 0 to ASource.FieldCount - 1 do
      ATarget.Fields[i].Value := ASource.Fields[i].Value;
    ATarget.Post;
    ASource.Next;
  end;
  ATarget.First;
end;

{+}
procedure CopyRecord(ASource, ATarget: TDataSet);
{
  Copies the data in like-named fields from one TDataSet to another.
  The data types of the fields are assumed to be compatible.
  The caller is responsible for setting up the Append/Edit beforehand and the
  call to Post after.
-}
var
  i: Integer;
  F: TField;
begin
  for i := 0 to ATarget.FieldCount - 1 do begin
    F := ASource.FindField(ATarget.Fields[i].FieldName);
    if Assigned(F) then
      ATarget.Fields[i].Value := F.Value;
  end;
end;

end.

