unit llSimpleSchedEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, Spin, llSimpleSched;

type
  TSchedEditForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    rbAlwaysOn: TRadioButton;
    rbTimeOfDay: TRadioButton;
    rbCycle: TRadioButton;
    StartTimeEdit: TEdit;
    StopTimeEdit: TEdit;
    CycleStartEdit: TEdit;
    OnEdit: TSpinEdit; // These allow 1..719 because
    OffEdit: TSpinEdit; // 1440/2 = 720 = the largest possible cycle schedule.
    btnOK: TButton;
    btnCancel: TButton;
    procedure FormDestroy(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FSched: TSimpleSchedule;
    function GetSchedStr: string;
  public
    { Public declarations }
    procedure SetSchedule(const AStart, AOn, AOff: Cardinal);
    procedure GetSchedule(out OStart, OOn, OOff: Cardinal);
    property ScheduleStr: string read GetSchedStr;
  published
    procedure FixTime(Sender: TObject);
    procedure TypeClick(Sender: TObject);
  end;

implementation

{$R *.dfm}

uses llDates, llMsgDlg, llStrings;

{ TSchedEditForm }

procedure TSchedEditForm.FormCreate(Sender: TObject);
begin
  { These are set in the form
  StartTimeEdit.Text := '07:00';
  StopTimeEdit.Text := '19:00';
  CycleStartEdit.Text := '00:00';
  OnEdit.Value := 15;
  OffEdit.Value := 15;
  }
  FSched := TSimpleSchedule.Create;
  rbAlwaysOn.Checked := True;
  TypeClick(Sender);
end;

procedure TSchedEditForm.FormDestroy(Sender: TObject);
begin
  FSched.Free;
end;

procedure TSchedEditForm.btnOKClick(Sender: TObject);
const
  cCaption = 'Invalid Schedule';
var
  iStart, iOn, iOff: Cardinal;
begin
  GetSchedule(iStart, iOn, iOff);
  if FSched.SetSchedule(iStart, iOn, iOff) then
    ModalResult := mrOk
  else if rbTimeOfDay.Checked then begin
    MsgError('Start and Stop may not be the same.', cCaption, Self);
    StartTimeEdit.SetFocus;
  end else begin
    MsgError('On + Off must be an even divisor of 1440 (minutes per day)' +
             #13#10 + 'so the schedule can start at the same time each day.',
             cCaption, Self);
    OnEdit.SetFocus;
  end;
end;

procedure TSchedEditForm.FixTime(Sender: TObject);
var
  Ed: TEdit;
  sText: string;
  dtTime: TDateTime;
begin
  // Called OnExit on the time edits
  if Sender is TEdit then begin
    Ed := TEdit(Sender);
    sText := DigitsOnly(Ed.Text);
    sText := PadL(sText, 4, '0'); // Allow 5:00 to become 0500
    if TryStoT(sText, dtTime) then
      Ed.Text := FormatDateTime('hh:nn', dtTime)
    else
      Ed.Text := '00:00';
  end;
end;

function TSchedEditForm.GetSchedStr: string;
begin
  // Should only be called after ShowModal returns mrOk
  Result := FSched.ScheduleStr;
end;

procedure TSchedEditForm.GetSchedule(out OStart, OOn, OOff: Cardinal);
  function GetTime(const AEdit: TEdit): TDateTime;
  begin
    if not TryStoT(DigitsOnly(AEdit.Text), Result) then
      Result := 0;
  end;
var
  dtStart, dtStop: TDateTime;
begin
  if rbAlwaysOn.Checked then begin
    OStart := 0;
    OOn := MinsPerDay;
    OOff := 0;
  end else if rbTimeOfDay.Checked then begin
    dtStart := GetTime(StartTimeEdit);
    OStart := TimeToMinutes(dtStart);
    dtStop := GetTime(StopTimeEdit);
    if dtStop < dtStart then
      dtStop := dtStop + 1;
    OOn := TimeToMinutes(dtStop - dtStart);
    OOff := MinsPerDay - OOn;
  end else begin // Cycle
    OStart := TimeToMinutes(GetTime(CycleStartEdit));
    OOn := OnEdit.Value;
    OOff := OffEdit.Value;
  end;
end;

procedure TSchedEditForm.SetSchedule(const AStart, AOn, AOff: Cardinal);
  function MakeTime(const AMinutes: Cardinal): string;
  var
    dt: TDateTime;
  begin
    dt := MinutesToTime(AMinutes);
    Result := FormatDateTime('hh:nn', dt);
  end;
begin
  if (AOff = 0) or not FSched.SetSchedule(AStart, AOn, AOff) then
    rbAlwaysOn.Checked := True
  else if (AOn + AOff) = MinsPerDay then begin
    rbTimeOfDay.Checked := True;
    StartTimeEdit.Text := MakeTime(AStart);
    StopTimeEdit.Text := MakeTime(AStart + AOn);
  end else begin
    rbCycle.Checked := True;
    CycleStartEdit.Text := MakeTime(AStart);
    OnEdit.Value := AOn;
    OffEdit.Value := AOff;
  end;
  TypeClick(nil);
end;

procedure TSchedEditForm.TypeClick(Sender: TObject);
begin
  // Called OnClick on the radio buttons
  StartTimeEdit.Enabled := rbTimeOfDay.Checked;
  StopTimeEdit.Enabled := rbTimeOfDay.Checked;
  CycleStartEdit.Enabled := rbCycle.Checked;
  OnEdit.Enabled := rbCycle.Checked;
  OffEdit.Enabled := rbCycle.Checked;
end;

end.
